\name{extract.sims.method}
\alias{extract.sims.method}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Extracts the p-values or q-values from a SetMethodsSims object
for a specific method.
}
\description{
This function is used to obtain a single data frame with the p-values or
q-values from one of the specific gene-set analysis methods, from a
\code{SetMethodsSims} object which has the results from simulated datasets.
}
\usage{
extract.sims.method(object, method)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
  Object of the class \code{SetMethodsSims}.
}
  \item{method}{
  Character string giving the method used for extraction, and whether p-values
  or q-values are extracted. The string should be one of the column names
  of the data frame resulting from the \code{do.gene.set.analysis} function.  
}
}
\value{
An object of the class \code{SetMethodsSims}. See
\code{SetMethodsSims} for more details.
}
\references{
Boca S.M., Kinzler K., Velculescu V.E., Vogelstein B.,
Parmigiani G.
Patient-oriented gene-set analysis for cancer mutation data.
\emph{Submitted}, 2010.
}
\author{
Simina M. Boca, Giovanni Parmigiani.
}

\seealso{
\code{SetMethodsSims-class},
\code{sim.data.p.values},
\code{do.gene.set.analysis}
}
\examples{
\dontrun{
##Note that this takes a few minutes to run:
library(KEGG.db)
data(Parsons)
data(ID2name)

set.seed(831984)

resultsSim <- 
    sim.data.p.values(EventsBySample = EventsBySampleBrain,
                      Mutations = MutationsBrain,
                      GeneSizes = GeneSizes08,
                      Coverage = CoverageBrain,
		      GeneSets = KEGGPATHID2EXTID[c("hsa05213", 
                      "hsa05223", "hsa00250")],
		      ID2name = ID2name,
                      nr.iter = 2,
                      pass.null = TRUE,
                      perc.samples = c(75, 95),
                      spiked.set.sizes = c(50),
                      show.iter = TRUE,
                      gene.method = FALSE,
                      perm.null.method = TRUE,
                      perm.null.het.method = FALSE,
                      pass.null.method = TRUE,
                      pass.null.het.method = FALSE)

resultsSim

extract.sims.method(resultsSim, "p.values.perm.null")
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
